# Copyright 2019, Oath Inc.
# Licensed under the terms of the Apache 2.0 license.  See the LICENSE file in the project root for terms
"""Exception classes"""


class VersionError(Exception):
    """An error has occurred generating a version"""
